Package: dandelionR
Title: Single-cell Immune Repertoire Trajectory Analysis in R
Version: 1.3.0
Authors@R: 
    c(person(given="Jiawei", family="Yu", email = "jiawei.yu@uq.edu.au", role = c("aut"), comment = c(ORCID = "0009-0005-9170-7881")),
      person(given="Nicholas", family="Borcherding", email = "borcherding@wustl.edu", role = c("aut"), comment = c(ORCID = "0000-0003-1427-6342")),
      person(given="Kelvin", family="Tuong", email = "z.tuong@uq.edu.au", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-6735-6808")))
Description: dandelionR is an R package for performing single-cell immune repertoire trajectory analysis, based on the original python implementation. It provides the necessary functions to interface with scRepertoire and a custom implementation of an absorbing Markov chain for pseudotime inference, inspired by the Palantir Python package.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
biocViews: Software, ImmunoOncology, SingleCell
Collate: 'check.R' 'constructMarkovChain.R' 'dandelionR.R' 'data.R'
        'determMultiscaleSpace.R' 'terminalStateFromMarkovChain.R'
        'differentiationProbabilities.R' 'filterCells.R' 'getPbs.R'
        'projectProbability.R' 'maxMinSampling.R' 'minMaxScale.R'
        'markovProbability.R' 'miloUmap.R' 'projectPseudotimeToCell.R'
        'setupVdjPseudobulk.R' 'splitCTgene.R' 'vdjPseudobulk.R'
Imports: BiocGenerics, bluster, destiny, igraph, MASS, Matrix, methods,
        miloR, purrr, rlang, S4Vectors, SingleCellExperiment, spam,
        stats, SummarizedExperiment, uwot, RANN
Suggests: BiocStyle, fields, knitr, rmarkdown, RColorBrewer, scater,
        scRepertoire, DelayedMatrixStats, slingshot, testthat
VignetteBuilder: knitr
URL: https://www.github.com/tuonglab/dandelionR/
BugReports: https://www.github.com/tuonglab/dandelionR/issues
Depends: R (>= 4.4.0)
git_url: https://git.bioconductor.org/packages/dandelionR
git_branch: devel
git_last_commit: 29dd237
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-06
NeedsCompilation: no
Packaged: 2025-11-06 22:07:57 UTC; biocbuild
Author: Jiawei Yu [aut] (ORCID: <https://orcid.org/0009-0005-9170-7881>),
  Nicholas Borcherding [aut] (ORCID:
    <https://orcid.org/0000-0003-1427-6342>),
  Kelvin Tuong [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6735-6808>)
Maintainer: Kelvin Tuong <z.tuong@uq.edu.au>
