% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constructMarkovChain.R
\name{.RANNinx}
\alias{.RANNinx}
\title{Calculate the weight adjacent matricks of knn graph and its index using RANN}
\usage{
.RANNinx(wp_data, knn.)
}
\arguments{
\item{wp_data}{Multi scale data of the waypoints}

\item{knn.}{Number of nearest neighbors for graph construction}
}
\value{
a list containing the weight adjacent matrix and index
}
\description{
Calculate the weight adjacent matricks of knn graph and its index using RANN
}
\keyword{internal}
